package neuroet;

public abstract class ActivationFunction implements java.io.Serializable
{
	public abstract double ComputeFunction(double x);
	public abstract double ComputeDerivative(double x);

	private static final double logsig(double x)
	{
          /*
          if (x < -45.0)//prevents overflow
            return 0.0;
          else if (x > 45.0)
            return 1.0;
          else
          */
            return 1.0 / (1.0 + Math.exp(-x));
	}

	public final static ActivationFunction LogSig = new ActivationFunction()
	{
		public double ComputeFunction(double x)
		{

			return ActivationFunction.logsig(x);
		}

		public double ComputeDerivative(double x)
		{
			double answer = ComputeFunction(x);
			return answer * (1.0 - answer);
		}

	};




	public final static ActivationFunction HyperbolicTangentSigmoid = new ActivationFunction()
	{

		public double ComputeFunction (double x)
		{

                  return (2.0 / (1.0 + Math.exp(-2.0 * x))) - 1.0;
			//return 2.0 * ActivationFunction.logsig(x * 0.5) - 1.0;  //do it this way to reduce chances of getting NaN
		}

		public double ComputeDerivative(double x)
		{
			double answer = ComputeFunction(x);
			return 1.0 - (answer * answer);
		}

	};

	public final static ActivationFunction PureLinear = new ActivationFunction()
	{

		public double ComputeFunction (double x)
		{
			return x;
		}

		public double ComputeDerivative(double x)
		{
			return 1;
		}

	};

	public final static ActivationFunction Exponential = new ActivationFunction()
	{

		public double ComputeFunction (double x)
		{
		//	System.out.println("Computed Exponential");
			return Math.exp(-x);
		}

		public double ComputeDerivative(double x)
		{
			return - Math.exp(-x);
		}

	};

	private static int nextOrdinal = 0;
	private final int ordinal = nextOrdinal++;

	private static final ActivationFunction[] VALUES = {LogSig, HyperbolicTangentSigmoid, PureLinear, Exponential};

	protected Object readResolve() throws java.io.ObjectStreamException{
		return VALUES[ordinal];
	}

}