package neuroet;
import java.io.PrintWriter;

public class HiddenLayer extends Layer
{

	protected Neuron[] ourNeurons;
        private final static long serialVersionUID= 6545621542L;

	/*HiddenLayer(int numNeurons, Layer pl)
	{
		if (numNeurons < 1)
			throw new IllegalArgumentException("Number of neurons in hidden layer < 1");

		ourNeurons = new Neuron[numNeurons];
		int inputs = pl.NumOutputs();
		previousLayer = pl;

		for (int x = 0; x < ourNeurons.length; x++)
		{
			ourNeurons[x] = new Neuron(inputs, ActivationFunction.HyperbolicTangentSigmoid);
		}


	}*/

	HiddenLayer(Neuron[] neu, Layer pl)
	{
                if (neu.length < 1)
                   throw new IllegalArgumentException("Must have at least one argument");

		ourNeurons = neu;
		previousLayer = pl;
	}

	public Layer copy()
	{
		Neuron[] copys = new Neuron[ourNeurons.length];

		for (int x = 0; x < ourNeurons.length; x++)
		{
			copys[x] = ourNeurons[x].copy();
		}

		return new HiddenLayer(copys, previousLayer.copy());

	}



	public final void ModifyNetwork (int current, double[] vals)
	{

		final int gradSize = ourNeurons[0].SizeGradient();

		for (int x = 0; x < ourNeurons.length; x++)
		{
			ourNeurons[x].ModifyAll(current, vals);
			current += gradSize;

		}


		previousLayer.ModifyNetwork(current, vals);



	}


	public double[][] GetWeights()
	{
		double[][] answer = new double[ourNeurons.length][];

		for (int x = 0; x < answer.length; x++)
		{
			answer[x] = ourNeurons[x].GetWeights();
		}

		return answer;
	}

        public void PrintNetworkWeights(PrintWriter pw)
        {
          previousLayer.PrintNetworkWeights(pw);
          pw.println();
          double[][] toPrint = GetWeights();

          pw.println(toPrint[0].length);
          Utilities.PrintDoubleMatrix(pw, toPrint);
        }

        public void PrintNetworkBiases(PrintWriter pw)
        {
          previousLayer.PrintNetworkBiases(pw);
          pw.println();
          Utilities.PrintVerticalDoubleArray(pw, GetBiases());
        }

	public final double[] GetBiases()
	{
		double[] answer = new double [ourNeurons.length];

		for (int x = 0; x < answer.length; x++)
		{
			answer[x] = ourNeurons[x].GetBias();
		}

		return answer;
	}


	public final void SetPreviousLayer(Layer l)
	{
		previousLayer = l;
	}

	public final double[] GetOutputs()
	{
		double[] answers = new double[ourNeurons.length];
		double[] input = previousLayer.GetOutputs();

		for (int x = 0; x < answers.length; x++)
		{
			answers[x] = ourNeurons[x].CalculateOutput(input);
		}


		return answers;
	}

	public final int NumParams()
	{
		return previousLayer.NumParams() + (ourNeurons[0].SizeGradient()) * ourNeurons.length;
	}


	public double[] CalculateGradient(int requested, double[] oldSensitivity)
	{
		double[] answer;

		final int gradPerNeuron = ourNeurons[0].SizeGradient();


	//	System.err.println("a");

		double[] sensitivity = CalculateSensitivity(oldSensitivity);
                double[][] weights = null;
		double[] weightsTimeSensitivity = null;

                if (!(this.previousLayer instanceof InputLayer)) //only perform these calculations if necessary
                {
                  weights = GetWeights();//dimensions are prevNeurons tall by currNeurons wide
                  weightsTimeSensitivity = new double[previousLayer.NumOutputs()];//we have one row per previous neuron


          //	System.err.println("c");



                  for (int x = 0; x < sensitivity.length; x++)
                  {
                    double s = sensitivity[x];
                    for (int y = 0; y < weightsTimeSensitivity.length; y++)
                    {
                      weightsTimeSensitivity[y] += s * weights[x][y];
                    }


                  }

                }

	//	System.err.println("d");

		answer = previousLayer.CalculateGradient(requested + ourNeurons.length * gradPerNeuron, weightsTimeSensitivity);

	//	System.err.println("e");

		for (int x = 0; x < ourNeurons.length; x++)
		{
			ourNeurons[x].CalculateGradient(sensitivity[x], requested + x * gradPerNeuron, answer);
		}

	//	System.err.println("b");

		return answer;

	}


	protected double[] CalculateSensitivity(double[] error)
	{
		double[] answers = new double[ourNeurons.length];


		//System.out.println(error.length + "\t" + ourNeurons.length);

		for (int x = 0; x < answers.length; x++)
		{
			answers[x] = ourNeurons[x].GetDerivative() * error[x];
		}

		return answers;
	}


	public void AdjustWeightsAndBiases(double[] oldSensitivity)
	{
		double[] sensitivity = CalculateSensitivity(oldSensitivity);
		double[][] weights = GetWeights();
		double[] weightsTimeSensitivity = new double[weights.length];

		for (int x = 0; x < sensitivity.length; x++)
		{
			weightsTimeSensitivity[x] = MultiplyVector(weights[x], sensitivity[x]);
			ourNeurons[x].AdjustWeightsAndBiases(sensitivity[x]);
		}


		previousLayer.AdjustWeightsAndBiases(weightsTimeSensitivity);

	}


	public final int NumOutputs()
	{
		return ourNeurons.length;
	}

	public final int NumInputs()
	{
		return ourNeurons[0].NumInputs();
	}

}

