package neuroet;

/**
 * <p>Title: Neural Net Classes</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: University of Washington</p>
 * @author Erik Tribou
 * @version 1.0
 */

/*
n = # of training cases
SSE = sum squared error
SSE / n = mean sum of squares
p = # of parameters (weights and biases) //get from NeuralNet.numParams
*/

public abstract class HiddenUnitOptimizer
{


  private static String[] names = {"SBC", "AICc"};

  public static String[] GetNames()
  {
    return (String[])names.clone();
  }

  public static HiddenUnitOptimizer ResolveName(String name)
  {
    if (name == names[0])
    {
      return SBCOptimizer;
    }
    else if (name == names[1])
    {
      return AICcOptimizer;
    }
    else
    {
      throw new IllegalArgumentException();
    }
  }

  //SBC = n (log SSE / n) + p (log(n))

  public static final HiddenUnitOptimizer SBCOptimizer = new HiddenUnitOptimizer()
  {
    public double GetScore(NeuralNet nn)
    {

      /*try{
      nn.DoIt(trainer,1.0);
      }catch (TrainingException te){}
      catch(InterruptedException ie){ throw new RuntimeException(ie.toString());}*/

      return GetFinalSBCScore(nn);

    }
  };


  //AiCc = n (log (SSE / n)) + (n + p) / (1 - (p + 2) / n)

  public static final HiddenUnitOptimizer AICcOptimizer = new HiddenUnitOptimizer()
  {
    public double GetScore(NeuralNet nn)
    {

      return GetFinalAICcScore(nn);
    }
  };



  protected HiddenUnitOptimizer()
  {

  }

  //Take already trained NNs
  public static double GetFinalAICcScore(NeuralNet nn)
  {
    double n = nn.NumTrainingCases();
    double p = nn.NumWeightsAndBiases();

    double SSE = nn.GetSSE();

    return n * Math.log(SSE / n) + (n + p) / (1.0 - (p + 2.0) / n);

  }

  public static double GetFinalSBCScore(NeuralNet nn)
  {
    double n = nn.NumTrainingCases();
    double p = nn.NumWeightsAndBiases();

    double SSE = nn.GetSSE();

    return n * Math.log(SSE / n) + p * Math.log(n);

  }

  public abstract double GetScore(NeuralNet nn); //lower should be better


}