package neuroet;

//An abstract layer...  Anything that's a layer should make a reasonable implementation of this
import java.io.PrintWriter;


public abstract class Layer extends Object implements java.io.Serializable
{

	protected Layer previousLayer; //will be null in an input layer

        private final static long serialVersionUID= 6565365981L;

	public abstract double[] GetOutputs();
	public abstract int NumOutputs();
	public abstract Layer copy();


	public abstract double[] CalculateGradient(int requested, double[] sensitivity);

	public abstract void AdjustWeightsAndBiases(double[] error);
	public abstract void ModifyNetwork (int current, double[] vals);


	public abstract int NumParams();

        public abstract void PrintNetworkBiases(PrintWriter pw);

        public abstract void PrintNetworkWeights(PrintWriter pw);

	public double[] CalculateJacobianRow(int req, double[] sensitivity)
	{
		return CalculateGradient(req, sensitivity);
	}

	public static double MultiplyVector(double[] one, double two)  //used in sum of weights times sensitivity
	{
		double answer = 0.0;

		for (int x = 0; x < one.length; x++)
		{
			answer += one[x] * two;
		}

		return answer;
	}

	final public InputLayer GetInputLayer()
	{
		if (previousLayer == null)
			return (InputLayer)this;
		else
			return (previousLayer.GetInputLayer());
	}


	final public void SetInputLayer(InputLayer l)
	{
		if (previousLayer instanceof HiddenLayer)
		{
			previousLayer.SetInputLayer(l);
		}
		else
		{
			HiddenLayer h = (HiddenLayer)this;

			if (h.NumInputs() != l.NumOutputs())
				throw new IllegalArgumentException("Number of inputs doesn't match next hidden layer");

			h.SetPreviousLayer(l);
		}

	}

}
