/*
	NeuralNet.java

	Author:			Erik Tribou
	Description:
*/

package neuroet;
import drasys.or.cont.PriorityQueue;
import java.util.*;
import java.io.*;
import javax.swing.*;
import drasys.or.cont.*;
import drasys.or.*;


public final class NeuralNet extends Object implements Serializable
{



	private OutputLayer ol;
        private static final int maxFailures = 5;//number of 2 second intervals to stop
        private DataSet data;
        private transient double currError = 0.0;

        private transient double SSE = Double.POSITIVE_INFINITY;
        private final static long serialVersionUID= 6549891353L;




        public double[][] MakeAsScaledInputs(File datax) throws IOException
        {
          return data.MakeAsScaledInputs(datax);
        }

        private NeuralNet(OutputLayer o, DataSet d)
        {
          ol = (OutputLayer)o.copy();
          data = d;
        }

        private NeuralNet copy()
        {
          return new NeuralNet(ol, data);
        }

        private synchronized void SetError(double e)
        {
          currError = e;
          //notifyAll();

        }



        public synchronized double GetError()//returns R^2
        {
          return currError;
        }

        public synchronized double GetSSE()//only valid after training is finished
        {
          return SSE;
        }

        public synchronized void SetSSE(double e)
        {
          SSE = e;
        }

	public static void RunNeuralNetOnData(File nnFile, File datax, File output) throws Exception
	{

		ObjectInputStream oi = new ObjectInputStream(new BufferedInputStream(new FileInputStream(nnFile)));
                NeuralNet nn;


		try{
		nn = (NeuralNet)oi.readObject();
		}finally{oi.close();}


		double[] guesses = nn.RunAndGetResults(new InputLayer(nn.MakeAsScaledInputs(datax)));
                double[][] matx= {guesses};

		PrintWriter predOut = new PrintWriter(new BufferedWriter(new FileWriter(output)));

                matx = ArrayUtilities.RotateMatrix(matx);

		try
		{
                  Utilities.PrintDoubleMatrix(predOut,matx);
		}finally{predOut.close();}
	}




	private void TrainNetwork(Trainer t, double[] answers, double threshold) throws TrainingException, InterruptedException
	{

		if (threshold < 0.0 || threshold > 1.0)
			throw new IllegalArgumentException("Illegal Threshold Value: " + threshold);





		//double[] fullAnsw = data.RevertOutputs(answers);


                final double ssyy = ArrayUtilities.CalculateSSYY(answers);//ArrayUtilities.CalculateSSYY(fullAnsw);


                SetError(0.0);


                int bad = 0;

                double lastRS = 0.0;

                int numChecks = 0;

                double minImprove = .001;

                Checker c = new Checker(2000);
                c.setPriority(Math.min(Thread.currentThread().getPriority() + 1, Thread.MAX_PRIORITY));
                c.start();
                int runs = 0;
                try{

		for (;;)
		{



                        double err = t.Train(answers);


                        runs++;

			if (c.CanGo())
			{

                          numChecks++;

                          if (numChecks == 90)//if we've spent 3 minutes, change minImprove
                          {
                            minImprove = 0.01;
                            bad = 0;
                          }

                          //double err = this.RunNetworkWithAverageError(ol, answers);//ArrayUtilities.SumOfSquares(RunAndGetResults(ol.GetInputLayer()), fullAnsw);

                                double currentRS = 1.0 - err / ssyy;//used to just be rs


                                SetError(currentRS);
                                //System.err.println(currentRS);



                                if ((lastRS + minImprove) > currentRS)
                                {
                                  bad++;
                                }
                                else
                                {
                                  bad = 0;
                                }


                                lastRS = currentRS;



				if (currentRS > threshold)// ? use >=
					break;
                                if (Thread.interrupted())
                                   throw new InterruptedException("Train Network has been cancelled");
                                if (bad > maxFailures)
                                   throw new TrainingException("Error function is no longer improving");

			}

		}

                }finally{
                  //System.out.println(runs);
                double err = this.RunNetworkWithAverageError(ol, answers);//ArrayUtilities.SumOfSquares(RunAndGetResults(ol.GetInputLayer()), fullAnsw);
                double currentRS = 1.0 - err / ssyy;//used to just be rs
                SetError(currentRS);
                SetSSE(err);
                c.Stop();
                }


	}


	//The priority queue will have the highest error of the top 10 in its head,
	//we compare each error to this and if error is lower we remove head and stick
	//new error into queue w/ copy of network

        //also note that it handles printing data

	private void TrainNetworkWithBestGeneralization(int trainer, double threshold, File svDir) throws IOException, TrainingException, InterruptedException
	{

		if (threshold < 0.0 || threshold > 1.0)
			throw new IllegalArgumentException("Bad threshold:\t" + threshold);

		PriorityQueue best = new PriorityQueue(new CompareNumberReverse(),10);
		Trainer t;



		double[] trainingAnswers = data.GetTrainingOutput();
		double[] testingAnswers = data.GetTestingOutput();

		InputLayer inTraining = new InputLayer (data.GetTrainingInputs());
		InputLayer inTesting = new InputLayer (data.GetTestingInputs());





		t = Trainer.GetTrainer(trainer, ol);




                Checker c = new Checker(2000);
                c.setPriority(Math.min(Thread.currentThread().getPriority() + 1, Thread.MAX_PRIORITY));
                c.start();

		//double[] fullAnsw = data.GetTrainingOutputUnscaled();
                //double[] fullTesting = data.GetTestingOutputUnscaled();


                double rs = 0.0;
                double testError = 0.0;
                SetError(rs);

                final double trainingSSYY = ArrayUtilities.CalculateSSYY(trainingAnswers);//ArrayUtilities.CalculateSSYY(fullAnsw);
                final double testingSSYY = ArrayUtilities.CalculateSSYY(testingAnswers);//ArrayUtilities.CalculateSSYY(fullTesting);
                //System.err.println(testingSSYY);
                int bad = 0;
                double lastRS = 0.0;
                //String lastRS = "0.0";

                try{
                  do
                  {
                          ol.SetInputLayer(inTraining);
                          //il = inTraining;
                          double err = t.Train(trainingAnswers);



                          if (c.CanGo())
                          {


                                  //double err = ArrayUtilities.SumOfSquares(RunAndGetResults(inTraining), fullAnsw);
                                  double currentRS = 1.0 - err/trainingSSYY;

                                  //rs = ArrayUtilities.CalculateRSquared(RunAndGetResults(inTraining),fullAnsw);

                                  SetError(currentRS);

                                  //String currentRS = nf.format(rs);

                                  if ((lastRS + .001) > currentRS)
                                  {
                                    bad++;
                                  }
                                  else
                                  {
                                    bad = 0;
                                  }

                                  lastRS = currentRS;



                                  if (Thread.interrupted())
                                     throw new InterruptedException("Train Network with best generalization has been interrupted");

                                  if (bad > maxFailures)
                                     throw new TrainingException("Error function no longer improving");
                          }





                          testError = 1.0 - ArrayUtilities.SumOfSquaresError(this.RunAndGetResultsUnscaled(inTesting), testingAnswers)/testingSSYY;

                          //System.err.println(ArrayUtilities.CalculateRSquared(this.RunAndGetResultsUnscaled(inTesting),testingAnswers) + "\t" + testError);
                          //testError = 1.0 - ArrayUtilities.SumOfSquares(RunAndGetResults(inTesting),fullTesting) / testingSSYY;


                          //System.out.println(testError);

                          if (best.size() < 10)
                          {
                                  //double err = this.RunNetworkWithAverageError(ol, inTraining);//ArrayUtilities.SumOfSquares(RunAndGetResults(inTraining), fullAnsw);
                                  rs = 1.0 - err/trainingSSYY;
                                  ErrorRecord er = new ErrorRecord(rs,testError);
                                  best.insert(er,this.copy());
                          }
                          else if (((ErrorRecord)(best.getHead().getFirst())).testError < testError)
                          {
                                  //double err = ArrayUtilities.SumOfSquares(RunAndGetResults(inTraining), fullAnsw);
                                  rs = 1.0 - err/trainingSSYY;
                                  ErrorRecord er = new ErrorRecord(rs,testError);
                                  best.popHead();
                          //	System.out.println(best.popHead().getFirst());
                                  best.insert(er, this.copy());
                          }




                  }while (GetError() < threshold);

                }catch(TrainingException te){System.err.println("Can't train anymore");}
                catch(InterruptedException ie){ie.printStackTrace();System.err.println("Still running");}
                finally{

                  double err = RunNetworkWithAverageError(ol, trainingAnswers);//ArrayUtilities.SumOfSquares(RunAndGetResults(inTraining), fullAnsw);
                  double currentRS = 1.0 - err/trainingSSYY;
                  SetError(currentRS);
                  SetSSE(err);
                  c.Stop();
                }


                //SetError(ArrayUtilities.CalculateRSquared(RunAndGetResults(inTraining),fullAnsw));

                PrintWriter datOut = new PrintWriter(new FileWriter(new File(svDir,"Results.txt")));
		ObjectOutputStream oos;

		datOut.println("Rank\tTraining R-Squared\tTesting R-Squared\tValidation R-Squared");
		PairI currentPair;
		int rank = best.size();


                while (rank > 0)
                {
                        File outFolder = new File(svDir, "NN #" + rank);

                        if (!outFolder.mkdir())
                          throw new IOException("Couldn't make results folder: " + outFolder);

			currentPair = best.popHead();
                        NeuralNet toWrite = (NeuralNet)currentPair.getSecond();
			datOut.println(rank + ":\t" + currentPair.getFirst() + "\t" + toWrite.ValidateData());


                        toWrite.PrintResults(outFolder);



                        rank--;

		}

		datOut.println();
		datOut.println("Last(Best) Training\t" + GetError() + "\t" + testError + "\t" + this.ValidateData());

                File last = new File(svDir, "Final Run");
                last.mkdir();
                PrintResults(last);


		datOut.close();



	}





        public NeuralNet(DataSet ds, NeuronMaker nm)
        {
          data = ds;

          InputLayer il = new InputLayer(data.GetTrainingInputs());
          HiddenLayer hl = new HiddenLayer(nm.GetHiddenNeurons(il.NumOutputs()), il);

          ol = new OutputLayer(nm.GetOutputNeuron(hl.NumOutputs()), hl);
        }


        public int NumWeightsAndBiases()
        {
          return this.ol.NumParams();
        }

        public int NumTrainingCases()
        {
          return this.ol.GetInputLayer().NumIterations();
        }

        public String[] GetInputNames()
        {
          return data.GetInputNames();
        }

        public Runnable GetTrainWithBestGeneralization(final int trainer, final double threshold, final File svDir)
        {
          return new Runnable()
          {
            public void run()
            {

              try{
                TrainNetworkWithBestGeneralization(trainer, threshold, svDir);
              }catch(Exception e){throw new RuntimeException(e.toString());}

            }
          };
        }


        public Runnable GetGenericTrain(final int trainer, final double threshold, final File svDir)
        {
          return new Runnable()
          {
            public void run()
            {
              try{

                try{
                  DoIt(trainer, threshold);
                }catch(TrainingException te){}

                PrintResults(svDir);

              }catch(Exception e){throw new RuntimeException(e.toString());}
            }
          };
        }


        public void DoIt(int trainer, double threshold) throws TrainingException, InterruptedException
        {
            TrainNetwork(Trainer.GetTrainer(trainer,ol),data.GetTrainingOutput(),threshold);
        }






	//Runs network and returns error for each answer


	static public double[] RunNetwork (OutputLayer ol, double[] answers)
	{
		double[] errors = new double[answers.length];

		for (int x = 0; x < errors.length; x++)
		{
			errors[x] = answers[x] - ol.GetOutput();
		}

		return errors;

	}




	static public double[] RunNetworkAndGetAverageGradient(OutputLayer ol, double[] answers)
	{

		double[][] ans = new double[answers.length][];
		double[] error = new double[1];

                //double[] toRet = new double[ol.NumParams()];//the code to add all the gradients together

		for (int x = 0; x < ans.length; x++)
		{
			error[0] = answers[x] - ol.GetOutput();
			ans[x] = ol.CalculateGradient(error);
                        //toRet = ArrayUtilities.AddVectors(toRet, ans[x]);
		}

		return ArrayUtilities.AverageDoubleArrays(ans);
                //return toRet;

	}

	//Runs network with mods and returns SSE

	static public double RunWithModsAndAverageError(OutputLayer ol, double[] answers, double multiplier, double[] mods)
	{
		OutputLayer toRun = (OutputLayer)ol.copy();

		toRun.ModifyNetwork(ArrayUtilities.MultiplyArray(multiplier, mods));



		return ArrayUtilities.SumOfSquares(RunNetwork(toRun, answers));


	}

	static public double RunNetworkWithAverageError(OutputLayer ol, double[] answers)
	{
		return ArrayUtilities.SumOfSquares(RunNetwork(ol, answers));
	}

	static public double RunWithModsAndAverageError(OutputLayer ol, double[] answers, double[] mods)
	{
		OutputLayer toRun = (OutputLayer)ol.copy();

		toRun.ModifyNetwork(mods);

		return ArrayUtilities.SumOfSquares(RunNetwork(toRun, answers));


	}

	//prints a bunch of data to the specified directory

	public void PrintResults(File directory) throws IOException
	{


                PrintWriter biasOut = new PrintWriter(new BufferedWriter(new FileWriter(new File(directory,"biases.txt"))));
		PrintWriter predOut = new PrintWriter(new BufferedWriter(new FileWriter(new File(directory,"predictions.txt"))));

		PrintWriter wgtOut = new PrintWriter(new BufferedWriter(new FileWriter(new File(directory,"weights.txt"))));


		ObjectOutputStream nn = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(new File(directory, "NN.ser"))));

		try{

			double[] guesses = RunAndGetResults(new InputLayer(data.GetScaledInputs()));
                        double[] origOutputs = data.GetOriginalOutput();

                        predOut.println("#" + "\t" + "Prediction" + "\t" + "Actual");
                        predOut.println();

			for (int x = 0; x < guesses.length; x++)
			{
				predOut.println (x + "\t" + guesses[x] + "\t" + origOutputs[x]);
			}

			predOut.println();
			predOut.println("R-Squared: " + ArrayUtilities.CalculateRSquared(guesses, origOutputs));





                        ol.PrintNetworkWeights(wgtOut);




                        ol.PrintNetworkBiases(biasOut);



            nn.writeObject(this);//save this nn

        }finally {
          biasOut.close();
          predOut.close();

          wgtOut.close();
          nn.close();
          }


	}





    //returns the R-Squared of the validation set
    public double ValidateData()
    {
      double[] guesses = RunAndGetResultsUnscaled(new InputLayer(data.GetValidationInputs()));

      double[] answers = data.GetValidationOutput();

      //Utilities.PrintDoubleArray(System.err, guesses);
      //Utilities.PrintDoubleArray(System.err, answers);

      return ArrayUtilities.CalculateRSquared(guesses, answers);

    }

    //Takes scaled inputs, returns regular answer
    public double[] RunAndGetResults (InputLayer i)
    {
    	InputLayer backup = ol.GetInputLayer();

    	ol.SetInputLayer(i);
    	double[] answer = new double[i.NumIterations()];


    	for (int x = 0; x < answer.length; x++)
    	{
    		answer[x] = ol.GetOutput();
    	}

        ol.SetInputLayer(backup);

    	return data.RevertOutputs(answer);


    }

    public double[] RunAndGetResultsUnscaled (InputLayer i)
    {
            InputLayer backup = ol.GetInputLayer();

            ol.SetInputLayer(i);
            double[] answer = new double[i.NumIterations()];


            for (int x = 0; x < answer.length; x++)
            {
                    answer[x] = ol.GetOutput();
            }

        ol.SetInputLayer(backup);

        return answer;//no revert outputs


    }



/*	public double[] RunAndGetResultsNoScale(InputLayer i, DataSet data)
	{

		InputLayer backup = il;

    	il = i;
    	ol.SetInputLayer(il);
    	double[] answer = new double[il.NumIterations()];


    	for (int x = 0; x < answer.length; x++)
    	{
    		answer[x] = ol.GetOutput();
    	}

		il = backup;
		return answer;

	}*/








}

class Checker extends Thread
{
  private final int waitTime;
  private boolean go;
  private boolean stop;

  Checker (int millis)
  {
    waitTime = millis;
    go = false;
    stop = false;
  }

  public void run()
  {
    while(!GetStop())
    {
      try{

        sleep(waitTime);
        synchronized (this)
        {
          go = true;
          while (go && !stop)
          {
            this.wait();
          }
        }

      }catch(InterruptedException ie){}
    }



  }

  private synchronized boolean GetStop()
  {
    return stop;
  }

  public synchronized void Stop()
  {
    stop = true;
    this.notify();
  }

  public synchronized boolean CanGo()
  {
    boolean val = go;
    if (val)
    {
      go = false;
      this.notify();
    }

    return val;
  }


}

class ErrorRecord extends Number
{
	public final double trainError;
	public final double testError;

	ErrorRecord(double trE, double teE)
	{
		trainError = trE;
		testError = teE;
	}

	public double doubleValue()
	{
		return testError;
	}

	public float floatValue()
	{
		return (float)testError;
	}

	public int intValue()
	{
		return (int)testError;
	}

	public long longValue()
	{
		return (long)testError;
	}
	public String toString()
	{
		return trainError + "\t" + testError;
	}
}

