package neuroet;
import java.util.*;
import drasys.or.cont.PriorityQueue;

/* Implements a Neuron...  Keeps track of its bias, weights, previousInputs,
activation function, and the previous sum of inputs */


public class Neuron extends Object implements java.io.Serializable
{
	protected double bias; //protected just in case we ever want to extend neuron
	protected final double[] weights;
	protected double[] previousInputs;

	protected final static double lr = 0.1;
	protected final ActivationFunction AF;

	protected double lastSum;

        private final static long serialVersionUID= 156653268L;

	Neuron(int numInputs, ActivationFunction activ)
	{




		weights = new double[numInputs];
                //bias = 0.0;

                bias = Utilities.rGen.nextGaussian();

		for (int x = 0; x < numInputs; x++)
		{
			weights[x] = Utilities.rGen.nextGaussian();
		}


		AF = activ;
		lastSum = Double.NaN;
	}

	protected Neuron (double[] ws, double bs, ActivationFunction activ)
	{
		bias = bs;
		weights = ws;
		AF = activ;
		lastSum = Double.NaN;
	}

	double[] GetWeights()
	{
		return weights;
	}



	public void ModifyAll(int start, double[] mods)
	{

		for (int x = 0; x < weights.length; x++)
		{
			weights[x] += mods[start + x];
		}
		bias += mods[start + weights.length];

	}

	public final void CalculateJacobian(double sensitivity, int position, double[] toStickInto)
	{
		CalculateGradient(sensitivity, position, toStickInto);
	}


	public double GetBias()
	{
		return bias;
	}

	public final double CalculateOutput(double[] theInput)
	{
		previousInputs = theInput;
		double sum = 0.0;

		for (int x = 0; x < weights.length; x++)
		{
			sum += theInput[x] * weights[x];
		}

		sum += bias;

		lastSum = sum;

		return AF.ComputeFunction(sum);
	}


	public void AdjustWeightsAndBiases(double sensitivity)//used in standard backprop
	{

		for (int x = 0; x < weights.length; x++)
		{
			weights[x] -= lr * sensitivity * previousInputs[x];
		}

		bias -= lr * sensitivity;
	}


	public void CalculateGradient(double sensitivity, int position, double[] toStickInto)  //gives us the gradient
	{


		for (int x = 0; x < weights.length; x++)
		{
			toStickInto[position++] =  sensitivity * previousInputs[x];
		}

		toStickInto[position] = sensitivity;//don't need to increment position


	}

	public final int NumInputs()
	{
		return weights.length;
	}

	public int SizeGradient()
	{
		return weights.length + 1;
	}

	public final double GetDerivative()
	{
		return AF.ComputeDerivative(lastSum);
	}

	public Neuron copy()
	{
		//double[] answer = new double[weights.length];
		//System.arraycopy(weights, 0, answer, 0, answer.length);

		return new Neuron((double[]) weights.clone(), bias, AF);
	}
}
