package neuroet;
import java.io.PrintWriter;

public final class OutputLayer extends Layer
{


	private Neuron output;
        private final static long serialVersionUID= 954132320L;

	/*OutputLayer(Layer pl)
	{
		previousLayer = pl;
		output = new Neuron(pl.NumOutputs(), ActivationFunction.PureLinear);
	}*/

	OutputLayer(Neuron op, Layer pl)
	{
		output = op;
		previousLayer = pl;

	}

	public Layer copy()
	{
		return new OutputLayer(output.copy(), previousLayer.copy());
	}


        public void PrintNetworkWeights(PrintWriter pw)
        {
          previousLayer.PrintNetworkWeights(pw);
          pw.println();
          Utilities.PrintDoubleArray(pw, GetWeights());

        }

        public void PrintNetworkBiases(PrintWriter pw)
        {
          previousLayer.PrintNetworkBiases(pw);
          pw.println();
          pw.println(GetBias());

        }

	public double[] CalculateJacobianRow(int requested, double[] error)
	{

		double sensitivity = -output.GetDerivative();

		double[] weights = GetWeights();
		double[] weightsTimeSensitivity = new double[weights.length];


		for (int x = 0; x < weightsTimeSensitivity.length; x++)
		{
			weightsTimeSensitivity[x] = sensitivity * weights[x];  //need to compute before changing weights
		}


		double[] answer = previousLayer.CalculateJacobianRow(requested + output.SizeGradient(),weightsTimeSensitivity);

		output.CalculateJacobian(sensitivity,requested,answer);

		return answer;

	}

	public double[] CalculateJacobianRow(double[] error)
	{
		return CalculateJacobianRow(0, error);

	}

	public void ModifyNetwork (int current, double[] vals)
	{

		output.ModifyAll(current,vals);
		current += output.SizeGradient();

		previousLayer.ModifyNetwork(current,vals);



	}

	public void ModifyNetwork (double[] vals)
	{
		ModifyNetwork(0, vals);
	}

	public double[] GetOutputs()
	{
		return new double[]{GetOutput()};
	}

	public double GetOutput()
	{
		return output.CalculateOutput(previousLayer.GetOutputs());
	}

	public int NumOutputs()
	{
		return 1;
	}

	public double[] GetWeights()
	{
		return output.GetWeights();
	}

	public double[] CalculateGradient(int requested, double[] error)
	{
		//double[] answer;
		double sensitivity = -output.GetDerivative() * error[0];

		double[] weights = GetWeights();
		double[] weightsTimeSensitivity = new double[weights.length];

		for (int x = 0; x < weightsTimeSensitivity.length; x++)
		{
			weightsTimeSensitivity[x] = sensitivity * weights[x];  //need to compute before changing weights
		}


		double[] answer = previousLayer.CalculateGradient(requested + output.SizeGradient(),weightsTimeSensitivity);


		output.CalculateGradient(sensitivity, requested ,answer);


		return answer;


	}

	public double[] CalculateGradient(double[] error)
	{
		return CalculateGradient(0, error);
	}

	public int NumParams()
	{
		return previousLayer.NumParams() + output.SizeGradient();
	}

	public double GetBias()
	{
		return output.GetBias();
	}

	public void AdjustWeightsAndBiases(double[] error)
	{

		double[] weights = GetWeights();
		double[] weightsTimeSensitivity = new double[weights.length];
		double sensitivity = -2.0 * output.GetDerivative() * error[0];



		for (int x = 0; x < weightsTimeSensitivity.length; x++)
		{
			weightsTimeSensitivity[x] = sensitivity * weights[x];  //need to compute before changing weights
		}


		output.AdjustWeightsAndBiases(sensitivity);
		previousLayer.AdjustWeightsAndBiases(weightsTimeSensitivity);


	}

}