package neuroet;


public abstract class Scaler implements java.io.Serializable
{

        private final static long serialVersionUID= 865412305L;

	public final double[] ScaleArray(double[] ar)
	{
		double[] answer = new double[ar.length];

		for (int x = 0; x < answer.length; x++)
		{
			answer[x] = ScaleSingle(ar[x]);
		}

		return answer;
	}

	public final double[] RevertArray(double[] ar)
	{
		double[] answer = new double[ar.length];

		for (int x = 0; x < answer.length; x++)
		{
			answer[x] = RevertSingle(ar[x]);
		}

		return answer;
	}

	public abstract double ScaleSingle(double sing);


	public abstract double RevertSingle(double sing);



	private static final Scaler dn = new DoNothingScaler();

	static public Scaler GetDoNothingScaler()
	{
		return dn;
	}

	static public Scaler GetLinearScaler(double[] t)
	{
		return new LinearScalingScaler(t);
	}

	static public Scaler GetLinearScaler(double[] t, double min, double max)
	{
		return new LinearScalingScaler(t, min, max);
	}

	static public Scaler GetStandardNormalScaler(double[] t)
	{
		return new StandardNormalDistributionScaler(t);
	}

}

class DoNothingScaler extends Scaler
{
        private final static long serialVersionUID= 46546321L;

	public double ScaleSingle(double ar)
	{
		return ar;
	}

	public double RevertSingle(double ar)
	{
		return ar;
	}

	public Object readResolve() throws java.io.ObjectStreamException
	{
		return Scaler.GetDoNothingScaler();  //enforce singleton
	}

}


class LinearScalingScaler extends Scaler
{

	private double sLow;
	private double sRange;
	private double oLow;
	private double oRange;

        private final static long serialVersionUID= 98784632L;

	LinearScalingScaler(double[] toWorkOn)
	{
		this (toWorkOn, -1.0, 1.0);
	}

	LinearScalingScaler(double[] toWorkOn, double min, double max)
	{
		if (max < min)
			throw new IllegalArgumentException("Max should be greater than min");

		sLow = min;
		sRange = max - min;
		oLow = ArrayUtilities.FindMin(toWorkOn);
		oRange = ArrayUtilities.FindMax(toWorkOn) - oLow;



	}

	public double ScaleSingle(double ar)
	{

		return sLow + sRange * ((ar - oLow) / oRange);

	}


	public double RevertSingle(double ar)
	{
		return oLow + oRange * ((ar - sLow) / sRange);
	}

}

class StandardNormalDistributionScaler extends Scaler  // produces a distribution with mean 0 and standard dev 1
{

        private final static long serialVersionUID= 686463216L;
	private double stdDev;
	private double mean;

	StandardNormalDistributionScaler(double[] ar)
	{
		mean = ArrayUtilities.CalculateMean(ar);
		stdDev = ArrayUtilities.CalculateStdDev(ar);

	}

	public double ScaleSingle (double ar)
	{

		return (ar - mean) / stdDev;

	}

	public double RevertSingle (double ar)
	{
		return (ar * stdDev) + mean;
	}
}

