package neuroet;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.util.List;
/*  Various Utilities mostly used for IO */

public final class Utilities
{
	public final static java.util.Random noRand = new java.util.Random(0);
	public final static java.util.Random rGen = new java.util.Random();

        private Utilities(){}

        public static long Factorial(int fact)
        {
          if (fact < 0)
            throw new IllegalArgumentException("Bad value for factorial: " + fact);

          long ans = 1;

          while (fact > 0)
          {
            ans *= fact;
            fact--;
          }

          return ans;

        }

        public static void ShowErrorMessage(Throwable t)//creates a non-blocking dialog
        {
          ShowErrorMessage(t.toString());

        }

        public static void ShowErrorMessage(final String t)//creates a non-blocking dialog
        {
          SwingUtilities.invokeLater(new Runnable()
           {
            public void run()
            {
              JOptionPane.showMessageDialog(null, t, "An Error Has Occurred", JOptionPane.ERROR_MESSAGE);
            }

           });




        }

        public static int Choose(int top, int bottom)//can easily overflow if abused
        {
          if (bottom == 0)
            return 1;

          return (int)(Utilities.MultiplyRange(top - bottom + 1, top) / Utilities.Factorial(bottom));


        }


        public static long MultiplyRange(int start, int end)
        {
          if (start > end)
          {
            throw new IllegalArgumentException("Start must not be greater than end");
          }

          long answer = 1;

          while (start <= end)
          {
            answer *= start;
            start++;
          }

          return answer;

        }

        public static void CenterWindow(Window w)
        {
          Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
          Dimension windSize = w.getSize();

          w.setLocation((screenSize.width / 2) - (windSize.width / 2), (screenSize.height / 2) - (windSize.height / 2));

        }

        public static void AlmostCenterWindow(Window w)
        {
          Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
          Dimension windSize = w.getSize();

          w.setLocation((screenSize.width / 2) - (windSize.width / 2) + rGen.nextInt() % 25, (screenSize.height / 2) - (windSize.height / 2) + rGen.nextInt() % 25);

        }

        public static void UpperRightWindow(Window w)
        {
          int sw = Toolkit.getDefaultToolkit().getScreenSize().width;



          w.setLocation(sw - w.getWidth(), 0);
        }

	static char[] ReadWholeFile(File theFile) throws IOException
        {
            BufferedReader bf = new BufferedReader(new FileReader(theFile));
            int len = (int)theFile.length();
            if (theFile.length() != len || !theFile.exists())
                    throw new IOException("File length longer than an int or doesn't exist: " + theFile);

            char[] theOutput = new char[len];
            bf.read(theOutput, 0, len);
            bf.close();

            return theOutput;

        }


        public static String[] GetAllCategories(StringTokenizer st)
        {

          String[] toReturn = new String[st.countTokens()];

          for (int x = 0; x < toReturn.length; x++)
          {
           toReturn[x] = st.nextToken();
          }

          return toReturn;
        }

        public static String ReadAll(BufferedReader i) throws IOException
        {
          StringBuffer toReturn = new StringBuffer(2000);

          for (int ch = i.read(); ch != -1; ch = i.read())
          {
            toReturn.append((char)ch);
          }

          return toReturn.toString();
        }


        public static void CreateDirectory (File dir) throws IOException
        {
          dir = new File(dir.getCanonicalPath());
          DeleteWholeTree(new File(dir.getCanonicalPath()));

          boolean success = dir.mkdir();

          if (!success)
             throw new IOException("Couldn't create a new directory");


        }

        public static File GenerateDirectory (File start)
        {
          for (;;)
          {
            int name = (int)(Math.random() * 1.0E7);
            File toCreate = new File (start, Integer.toString(name));

            if (toCreate.mkdir())
              return toCreate;
          }

        }

        private static File[] StringArrayToFileArray(File parent, String[] myFiles)
        {
          File[] toReturn = new File[myFiles.length];

          for (int x = 0; x < toReturn.length; x++)
          {
            toReturn[x] = new File(parent, myFiles[x]);
          }

          return toReturn;
        }


        public static void DeleteWholeTree(File del) throws IOException
        {
          if (!del.exists())
          {
             throw new IOException("Given Non-Existant File");
          }


          if (del.isFile())
          {
             boolean success = del.delete();
             if (success)
             {
                return;
             }
             else
             {
                throw new IOException("Couldn't delete file: " + del);
             }

          }

          if (del.isDirectory())
          {
            File[] f = StringArrayToFileArray(del, del.list());

            for (int x = 0; x < f.length; x++)
            {
              DeleteWholeTree(f[x]);
            }

            boolean success = del.delete();

            if (success)
            {
              return;
            }
            else
            {
              throw new IOException("Couldn't delete file: " + del);
            }


          }
          else
          {
            throw new IOException("Unexpected file type encountered: " + del);
          }


        }

	static void PrintDoubleMatrix (PrintWriter output, double[][] toPrint)
	{
		for (int x = 0; x < toPrint.length; x++)
		{
			PrintDoubleArray(output, toPrint[x]);
		}
	}

	static void PrintDoubleArray (PrintWriter output, double[] toPrint)
	{
		for (int x = 0; x < toPrint.length; x++)
		{
			output.print (toPrint[x] + "\t");
		}

		output.println();
	}

	static void PrintDoubleMatrix (PrintStream output, double[][] toPrint)//for the system streams
	{
		for (int x = 0; x < toPrint.length; x++)
		{
			PrintDoubleArray(output, toPrint[x]);
		}
	}

	static void PrintDoubleArray (PrintStream output, double[] toPrint)
	{
		for (int x = 0; x < toPrint.length; x++)
		{
			output.print (toPrint[x] + "\t");
		}

		output.println();
	}


    static void PrintObjectArray (PrintStream output, Object[] toPrint)
    {
        for (int x = 0; x < toPrint.length; x++)
        {
            output.println(toPrint[x]);
        }
    }

    static void PrintObjectArray (PrintWriter output, Object[] toPrint)
    {
        for (int x = 0; x < toPrint.length; x++)
        {
            output.println(toPrint[x]);
        }
    }

    static void PrintVerticalDoubleArray(PrintWriter output, double[] toPrint)
    {
    	for (int x = 0; x < toPrint.length; x++)
		{
			output.println (toPrint[x]);
		}

    }

    static void PrintVerticalDoubleArray(PrintStream output, double[] toPrint)
    {
    	for (int x = 0; x < toPrint.length; x++)
		{
			output.println (toPrint[x]);
		}


    }

    public static File QueryForSaveFile(Frame parent, String title, String suggested) throws FileNotFoundException
    {
    	FileDialog fd = new FileDialog(parent, title, FileDialog.SAVE);

    	fd.setFile(suggested);
    	fd.show();

    		if (fd.getFile() == null)
    			throw new FileNotFoundException ("No File Specified");

    	return (new File(fd.getDirectory(), fd.getFile()));
    }



    public static void FixMaximumSize (JComponent myComp)//sets a component's max height size to its preferred size (for box layout)
    {
    	Dimension d = myComp.getMaximumSize();


    	d.height = myComp.getPreferredSize().height;
        myComp.setMaximumSize(d);
    }

    public static double[][] ReadMatrix(File toRead) throws IOException
    {
      MatrixReader mr = new MatrixReader(toRead);
      double[][] answer = new double[mr.NumberLeft()][];

      for (int x = 0; x < answer.length; x++)
      {
        answer[x] = mr.GetNext();
      }

      return answer;

    }

    public static double[][] ReadMatrix(String toUse)
    {
      MatrixReader mr = new MatrixReader(toUse);
      double[][] answer = new double[mr.NumberLeft()][];

      for (int x = 0; x < answer.length; x++)
      {
        answer[x] = mr.GetNext();
      }

      return answer;
    }

    public static ImageIcon GetImageFromResource (String resource) throws IOException
    {
      BufferedInputStream is = new BufferedInputStream(Utilities.class.getResourceAsStream(resource));

      ByteArrayOutputStream baos = new ByteArrayOutputStream(5000);

      try{
        for (int read = is.read(); read != -1; read = is.read())
        {
            baos.write(read);
        }

    }finally{is.close(); baos.close();}


    byte[] buffer = baos.toByteArray();



    return new ImageIcon(buffer);


    }

    /*
    public static List QuickSelect(Collection l, Comparator c, int lowbound, int highbound)
    {
      List cop = new ArrayList(l);


      int last = l.size() - 1;

      if (lowbound > last)
      {
        return Collections.EMPTY_LIST;
      }

      if (highbound > last)
      {
        highbound = last + 1;
      }

      return QuickSelectDangerous(cop, c, lowbound, highbound);
    }


    private static List QuickSelectDangerous(List l, Comparator c, int lowbound, int highbound)
    {

      int size = l.size();

      if (lowbound == 0 && highbound == size)
      {
        Collections.sort(l, c);
      }
      else if (size < 11)
      {
        Collections.sort(l, c);
      }
      else //partition
      {
        Object median = MedianOfThree(c, l);

        int medianIndex = (median == l.get(0) ? 0 : (median == l.get(size - 1)) ? (size - 1): (size / 2);

        int i = -1;
        int j = size;

        do
        {

          while (c.compare(l.get(++i), median) < 0){}
          while(c.compare(median, l.get(--j)) <= 0) {}

          if( i < j )
          {
            Object temp = l.get(i);

            l.set(i, l.get(j));
            l.set(j, temp);
          }


        }while (i < j);

                //more stuff

                List leftSub = l.subList(0, i);//list contains elements 0 - i
                List rightSub = l.subList(i + 1, size); //list contains elements i + 1 - size

                if (lowbound <= i) //there is stuff in the left partition
                {
                  int leftSubSize = leftSub.size();

                  List lP = QuickSelectDangerous(leftSub, c, lowbound, null);


                  if (highbound > i)//is in right as well
                  {
                    List answer = new ArrayList(lP);
                    List rP = null;

                    answer.addAll(rP);
                    return answer;
                  }

                  return lp;

                }
                else //all stuff in right partition
                {
                  return QuickSelectDangerous(rightSub, c, lowbound, highbound);
                }

      }



      return l.subList(lowbound, highbound);

    }*/

    public static final Comparator compareToComparator = new Comparator()
    {
      public int compare (Object o1, Object o2)
      {
        return ((Comparable)o1).compareTo(o2);
      }

      public boolean equals (Object o)
      {
        return (o == compareToComparator);
      }
    };
/*
    public static Object MedianOfThree(Comparator comp, List l)
    {
      int size = l.size();

      if (size < 3)
        throw new IllegalArgumentException("Need a size of at least three to take a median of three");

      return MedianOfThree(comp, l.get(0), l.get(size - 1), l.get(size / 2));

    }

    public static Object MedianOfThree(Comparator comp, Object one, Object two, Object three)
    {
      Object temp = null;

      boolean oneTwo = (comp.compare(one, two) > 0);

      if (oneTwo != (comp.compare(one, three) > 0))
      {
        return one;
      }

      if (oneTwo != (comp.compare (three, two) > 0))
      {
        return two;
      }

      return three;

    }*/

    //not tested
    public static long IntPow(long l, int p)
    {
      if (p < 0)
      {
        throw new IllegalArgumentException("Need a non-negative value for p");
      }

      if (p == 0)
        return 1L;
      if (p == 1)
        return l;


      if (p % 2 == 0)
      {
        long answer = IntPow(l, p / 2);

        return (answer * answer);
      }
      else
      {
        return (l * IntPow(l, p - 1));
      }

    }

}


class MatrixReader
{
    StringTokenizer myTokenizer;

    MatrixReader(File toRead) throws IOException
    {
        this(new String(Utilities.ReadWholeFile(toRead)));
    }

    MatrixReader(String i)
    {
      myTokenizer = new StringTokenizer(i, "\r\n");
    }

    double[] GetNext()
    {


        if (myTokenizer.hasMoreElements())
        {

            StringTokenizer parser = new StringTokenizer(myTokenizer.nextToken());//eats whitespace
            double[] toReturn = new double[parser.countTokens()];

            for (int x = 0; x < toReturn.length; x++)
            {
                toReturn[x] = Double.parseDouble(parser.nextToken());
            }
            return toReturn;
        }


        throw new java.util.NoSuchElementException("No more strings to parse in matrix file");


    }

    int NumberLeft()
    {
        return myTokenizer.countTokens();
    }
}
